<?php
/****************************************************
             Moon Rise and Set
                 PHP class
      21 Dec 2011 Jiri Jozif jirkaj@beda.cz
*****************************************************
 Source:
 Oliver Montenbruck and Thomas Pfleger: "Astronomy on the Personal Computer", Springer
 http://www.stargazing.net/Kepler/moonrise.html
 http://bodmas.org/astronomy/riset.html
*****************************************************/
class MoonRiSet
{
    const ARC = 206264.8;
    const SIN_EPS = 0.3978; // sin+cos obliquity ecliptic (23d26m)
    const COS_EPS = 0.9175;
    const PREC = 24;    // precision

    public $earthLatitude;
    public $earthLongitude;
    public $earthTimezone;
    public $tdiff;
    public $transit;
    public $rise;
    public $set;
    public $rise2;
    public $set2;

    private $_sinEarthLatitude, $_cosEarthLatitude;
    private $_data = array();

    public function __construct($earthLatitude = false, $earthLongitude = false, $earthTimezone = false) {
        if ($earthLatitude === false)
            $this->earthLatitude = ini_get('date.default_latitude');
        else
            $this->earthLatitude = $earthLatitude;

        if ($earthLongitude === false)
            $this->earthLongitude = ini_get('date.default_longitude');
        else
            $this->earthLongitude = $earthLongitude;

        if ($earthTimezone === false)
            $this->earthTimezone = ini_get('date.timezone');
        else
            $this->earthTimezone = $earthTimezone;
        // set current day
        $this->setDate(date("Y", time()), date("n", time()), date("j", time()));
    }
    // set day
    public function setDate($year, $month, $day) {
        if ($year < 1583 or $year > 2500) return(false);

        $old_timezone = date_default_timezone_get();
        date_default_timezone_set($this->earthTimezone);

        // calculation day's table, begin+end time
        $t = $tb = mktime(0, 0, 0, $month, $day, $year);
        $te = mktime(24, 0, 0, $month, $day, $year);
        $this->tdiff = ($te - $tb) / self::PREC;
        $this->_sinEarthLatitude = $this->dsin($this->earthLatitude);
        $this->_cosEarthLatitude = $this->dcos($this->earthLatitude);

        $i = 0;
        while ($i <= self::PREC) {
            $this->_data[$i]["timestamp"] = $t;
            $jd = $this->getJulianDate($t);
            $LST = $this->getLST($jd); // Local Sidereal Time
            $this->_data[$i]["LST"] = $LST;
            list($RA, $de) = $this->miniMoon(($jd - 2451545.0) / 36525.0);
            $this->_data[$i]["RA"] = $RA;
            $HA = $LST - $RA;    // hour angle
            if ($HA > 12) $HA -= 24;
            $this->_data[$i]["HA"] = $HA;
            $this->_data[$i]["sAlt"] = $this->_sinEarthLatitude * $this->dsin($de) + $this->_cosEarthLatitude * $this->dcos($de) * $this->dcos(15*$this->_data[$i]["HA"]); // sinus Altitude
            $t += $this->tdiff;
            $i++;
        }
        // Moon transit
        list($this->transit["timestamp"], $this->transit["hhmm"], $this->transit["hh:mm"]) = $this->getTransit("HA");
        // Moon's rise and set
        list(
            $this->rise["timestamp"], $this->rise["hhmm"], $this->rise["hh:mm"],
            $this->set["timestamp"], $this->set["hhmm"], $this->set["hh:mm"],
            $this->rise2["timestamp"], $this->rise2["hhmm"], $this->rise2["hh:mm"],
            $this->set2["timestamp"], $this->set2["hhmm"], $this->set2["hh:mm"]
        ) = $this->getRiSet("sAlt", $this->dsin(0.125));

        date_default_timezone_set($old_timezone);
        return(true);
    }
    // PRIVATE //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    private function getTransit($o) {
        $fT = false;
        for ($i = 1; $i < self::PREC && !$fT; $i += 2) {
            if (($this->_data[$i-1][$o] < 0 && $this->_data[$i][$o] >= 0) || ($this->_data[$i][$o] <= 0 && $this->_data[$i+1][$o] > 0)) {
                list($nz, $z1, $z2, $xe, $ye) = $this->quad($this->_data[$i-1][$o], $this->_data[$i][$o], $this->_data[$i+1][$o]);
                $fT = true;
                $oTran = $this->_data[$i]["timestamp"] + $this->tdiff*$z1;
            }
        }
        if ($fT)
            list($transit["timestamp"], $transit["hhmm"], $transit["hh:mm"]) = $this->formatTime($oTran);
        else {
            $transit["timestamp"] = false;
            $transit["hhmm"] = "    ";
            $transit["hh:mm"] = "     ";
        }
        return array($transit["timestamp"], $transit["hhmm"], $transit["hh:mm"]);
    }
    private function getRiSet($o, $alt) {

        $fRise = $fSet = false;
        $fAbove = false;
        $oRise2 = $oSet2 = false;

        if ($this->_data[0][$o] > $alt)
            $fAbove = true;
        for ($i = 1; $i < self::PREC; $i += 2) {
            list($nz, $z1, $z2, $xe, $ye) = $this->quad($this->_data[$i-1][$o]-$alt, $this->_data[$i][$o]-$alt, $this->_data[$i+1][$o]-$alt);
            if ($nz == 1) {
                if ($this->_data[$i-1][$o] < $alt) {
                    if ($fRise === true)
                        $oRise2 = $this->_data[$i]["timestamp"] + $this->tdiff*$z1;
                    else {
                        $oRise = $this->_data[$i]["timestamp"] + $this->tdiff*$z1;
                        $fRise = true;
                    }
                }
                else {
                    if ($fSet === true)
                        $oSet2 = $this->_data[$i]["timestamp"] + $this->tdiff*$z1;
                    else {
                        $oSet = $this->_data[$i]["timestamp"] + $this->tdiff*$z1;
                        $fSet = true;
                    }
                }
            }
            elseif ($nz == 2) {
                if ($ye < 0.0) {
                    $oRise = $this->_data[$i]["timestamp"] + $this->tdiff*$z2;
                    $oSet = $this->_data[$i]["timestamp"] + $this->tdiff*$z1;
                }
                else {
                    $oRise = $this->_data[$i]["timestamp"] + $this->tdiff*$z1;
                    $oSet = $this->_data[$i]["timestamp"] + $this->tdiff*$z2;
                }
                $fRise = $fSet = true;
            }
        }
        // output
        if ($fRise === true || $fSet === true ) {
            if ($fRise === true) {
                list($rise["timestamp"], $rise["hhmm"], $rise["hh:mm"]) = $this->formatTime($oRise);
                list($rise2["timestamp"], $rise2["hhmm"], $rise2["hh:mm"]) = $this->formatTime($oRise2);
            }
            else {
                $rise["timestamp"] = false;
                $rise["hhmm"] = "    ";
                $rise["hh:mm"] = "     ";
            }
            if ($fSet === true) {
                list($set["timestamp"], $set["hhmm"], $set["hh:mm"]) = $this->formatTime($oSet);
                list($set2["timestamp"], $set2["hhmm"], $set2["hh:mm"]) = $this->formatTime($oSet2);
            }
            else {
                $set["timestamp"] = true;
                $set["hhmm"] = "    ";
                $set["hh:mm"] = "     ";
            }
        }
        else {
            if ($fAbove === true) { // continuously above horizon
                $rise["timestamp"] = $set["timestamp"] = true;
                $rise["hhmm"] = $set["hhmm"] = "****";
                $rise["hh:mm"] = $set["hh:mm"] = "**:**";
            }
            else { // continuously below horizon
                $rise["timestamp"] = $set["timestamp"] = false;
                $rise["hhmm"] = $set["hhmm"] = "----";
                $rise["hh:mm"] = $set["hh:mm"] = "--:--";
            }
        }
        // return
        if ($oRise2 !== false)
            return array(
                $rise["timestamp"], $rise["hhmm"], $rise["hh:mm"],
                $set["timestamp"], $set["hhmm"], $set["hh:mm"],
                $rise2["timestamp"], $rise2["hhmm"], $rise2["hh:mm"],
                false, false, false);
        elseif ($oSet2 !== false)
            return array(
                $rise["timestamp"], $rise["hhmm"], $rise["hh:mm"],
                $set["timestamp"], $set["hhmm"], $set["hh:mm"],
                false, false, false,
                $set2["timestamp"], $set2["hhmm"], $set2["hh:mm"]);
        else
            return array(
                $rise["timestamp"], $rise["hhmm"], $rise["hh:mm"],
                $set["timestamp"], $set["hhmm"], $set["hh:mm"],
                false, false, false,
                false, false, false);
    }

    // Low precision formulae for planetary position, Flandern & Pulkkinen
    // returns ra and dec of Moon to 5 arc min (ra) and 1 arc min (dec)
    // for a few centuries either side of J2000.0
    // Predicts rise and set times to within minutes for about 500 years
    private function miniMoon($T) {

        $l0 = $this->frac(0.606433 + 1336.855225 * $T);
        $l = 2*M_PI * $this->frac(0.374897 + 1325.552410 * $T);
        $ls = 2*M_PI * $this->frac(0.993133 + 99.997361 * $T);
        $d = 2*M_PI * $this->frac(0.827361 + 1236.853086 * $T);
        $f = 2*M_PI * $this->frac(0.259086 + 1342.227825 * $T);

        // perturbation
        $dl =  22640 * sin($l);
        $dl += -4586 * sin($l - 2*$d);
        $dl += +2370 * sin(2*$d);
        $dl +=  +769 * sin(2*$l);
        $dl +=  -668 * sin($ls);
        $dl +=  -412 * sin(2*$f);
        $dl +=  -212 * sin(2*$l - 2*$d);
        $dl +=  -206 * sin($l + $ls - 2*$d);
        $dl +=  +192 * sin($l + 2*$d);
        $dl +=  -165 * sin($ls - 2*$d);
        $dl +=  -125 * sin($d);
        $dl +=  -110 * sin($l + $ls);
        $dl +=  +148 * sin($l - $ls);
        $dl +=   -55 * sin(2*$f - 2*$d);

        $s = $f + ($dl + 412 * sin(2*$f) + 541*sin($ls)) / self::ARC;
        $h = $f - 2*$d;

        $n =   -526 * sin($h);
        $n +=   +44 * sin($l + $h);
        $n +=   -31 * sin(-$l + $h);
        $n +=   -23 * sin($ls + $h);
        $n +=   +11 * sin(-$ls + $h);
        $n +=   -25 * sin(-2*$l + $f);
        $n +=   +21 * sin(-$l + $f);

        $l_moon = 2 * M_PI * $this->frac($l0 + $dl / 1296000);
        $b_moon = (18520.0 * sin($s) + $n) / self::ARC;

        // convert to equatorial coords using a fixed ecliptic
        $cb = cos($b_moon);
        $x = $cb * cos($l_moon);
        $v = $cb * sin($l_moon);
        $w = sin($b_moon);
        $y = self::COS_EPS * $v - self::SIN_EPS * $w;
        $z = self::SIN_EPS * $v + self::COS_EPS * $w;
        $rho = sqrt(1.0 - $z*$z);
        $de = (180/M_PI) * atan($z / $rho);
        $RA = (24/M_PI) * atan($y / ($x + $rho));
        if ($RA < 0) $RA += 24;
        return(array($RA, $de));
    }
    // finds the parabola throuh the three points (-1,ym), (0,yz), (1, yp) and returns
    // the coordinates of the values of x where the parabola crosses zero (roots of the quadratic)
    // and the number of roots (0, 1 or 2) within the interval [-1, 1]
    private function quad($ym, $yz, $yp) {
        $z1 = $z2 = 0;
        $nz = 0;
        $a = 0.5 * ($ym + $yp) - $yz;
        $b = 0.5 * ($yp - $ym);
        $c = $yz;
        $xe = -$b / (2 * $a);
        $ye = ($a * $xe + $b) * $xe + $c;
        $dis = $b * $b - 4.0 * $a * $c;
        if ($dis > 0) {
            $dx = 0.5 * sqrt($dis) / abs($a);
            $z1 = $xe - $dx;
            $z2 = $xe + $dx;
            if (abs($z1) <= 1.0) $nz += 1;
            if (abs($z2) <= 1.0) $nz += 1;
            if ($z1 < -1.0) $z1 = $z2;
        }
        return(array($nz, $z1, $z2, $xe, $ye));
    }
    private function getJulianDate($t) {
        //return $t/86400 + 2440587.5; // only for 64bit && year > 1582
        $jd = gregoriantojd(gmdate("n", $t) ,gmdate("j", $t) , gmdate("Y", $t)) - 0.5;
        $jd += gmdate("H", $t)/24 + gmdate("i", $t)/1440 + gmdate("s", $t)/86400;
        return($jd);
    }
    // returns the local sidereal time (degree)
    private function getLST($jd) {
        $mjd = $jd - 2451545.0;
        $lst = $this->range(280.46061837 + 360.98564736629 * $mjd);
        return ($lst + $this->earthLongitude)/15;
    }
    // round time, return array(timestamp, "hhmm", "hh:mm")
    private function formatTime($t) {
        $t = (int)$t;
        $t0 = 60*(int)($t/60+0.5);
        if (date("j", $t) == date("j", $t0))
            $t = $t0;
        return array($t, date("Hi", $t), date("H:i", $t));
    }

    private function frac($x) {
        return($x - floor($x));
    }
    private function range($x) {
        return($x - 360.0 * (Floor($x / 360.0)));
    }
    private function dsin($x) {
        return(sin(deg2rad($x)));
    }
    private function dcos($x) {
        return(cos(deg2rad($x)));
    }
}
?>
